'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var t = _ref.types;


  var hasDirective = function hasDirective(node, directive) {
    return !!(node.body.directives && node.body.directives.some(function (_ref2) {
      var value = _ref2.value;
      return value.value == directive;
    }));
  };

  var getProgram = function getProgram(path) {
    return t.isProgram(path.node) ? path : getProgram(path.parentPath);
  };

  return {
    visitor: {
      FunctionDeclaration: function FunctionDeclaration(path) {
        /* All FunctionDeclaration need to be a
          VariableDeclaration of an FunctionExpression */

        var node = path.node;

        if (hasDirective(node, "no curry") || ignoredFunctionNames.includes(node.id.name)) {
          return; // All is already done
        }

        var fn = t.functionExpression(node.id, node.params, node.body);

        var next = t.variableDeclaration("const", [t.variableDeclarator(node.id, fn)]);

        path.replaceWith(next);
      },
      FunctionExpression: function FunctionExpression(path) {
        /* All FunctionExpression without a _magicCurrying CallExpression
          will be embraced by it */

        var node = path.node,
            parent = path.parentPath.node;

        if (hasDirective(node, "no curry") || node.id !== null && ignoredFunctionNames.includes(node.id.name) || t.isCallExpression(parent) && parent.callee.name === "_magicCurrying") {
          return; // All is already done
        }

        var next = t.callExpression(t.identifier("_magicCurrying"), [node]);

        path.replaceWith(next);
      },
      ArrowFunctionExpression: function ArrowFunctionExpression(path) {
        /* All ArrowFunctionExpression without a _magicCurrying CallExpression
          will be embraced by it */

        var node = path.node,
            parent = path.parentPath.node;

        if (hasDirective(node, "no curry") || t.isCallExpression(parent) && parent.callee.name === "_magicCurrying") {
          return; // All is already done
        }

        var next = t.callExpression(t.identifier("_magicCurrying"), [node]);

        path.replaceWith(next);
      },
      CallExpression: function CallExpression(path) {
        /* If at least one _magicCurrying CallExpression exist
          we need the _magicCurrying FunctionDeclaration */

        var node = path.node;

        if (node.callee.name !== "_magicCurrying" || path.scope.references["_magicCurrying"]) {
          return; // All is already done
        }

        var declar = buildMagicCurryingGlobalFunction();
        declar._blockHoist = 3;

        getProgram(path).unshiftContainer("body", [declar]);
      }
    }
  };
};

var _babelTemplate = require('babel-template');

var _babelTemplate2 = _interopRequireDefault(_babelTemplate);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var buildMagicCurryingGlobalFunction = (0, _babelTemplate2.default)('\n  function _magicCurrying(fn) {\n    return function _magicCurrying_fn() {\n      var params = Array.prototype.slice.call(arguments);\n      if (params.length >= fn.length) {\n        return fn.apply(this, params);\n      }\n      return function _magicCurrying() {\n        var next = Array.prototype.slice.call(arguments);\n        return _magicCurrying_fn.apply(this, params.concat(next));\n      };\n    };\n  }\n');

var ignoredFunctionNames = ['_magicCurrying', '_magicCurrying_fn', '_interopRequireDefault'];